C=======================================================================
C GMCALC v1.0: a calculator for the Georgi-Machacek model 
C (with the most general custodial-symmetry-invariant scalar potential)
C by Katy Hartling, Kunal Kumar, and Heather E. Logan
C http://people.physics.carleton.ca/~logan/gmcalc/
C========================================================================

      PROGRAM GMMG5
C Program to generate the MG5 param_card for the Georgi-Machacek model.
      IMPLICIT NONE
C Common blocks:
      DOUBLE PRECISION MU2SQ,MU3SQ,LAMBDA1,LAMBDA2,LAMBDA3,LAMBDA4,
     .     LAMBDA5,M1,M2
      COMMON/LPARAMS/MU2SQ,MU3SQ,LAMBDA1,LAMBDA2,LAMBDA3,LAMBDA4,
     .     LAMBDA5,M1,M2
      INTEGER UNIOK, BFBOK, POSMSQOK, MINOK, INPUTOK
      COMMON/FLAGS/UNIOK, BFBOK, POSMSQOK, MINOK, INPUTOK
      INTEGER INPUTMODE, SILENT
      COMMON/CONTROLS/INPUTMODE,SILENT
      DOUBLE PRECISION MH
      INTEGER INPUTSET
      COMMON/INPUT/MH,INPUTSET
      DOUBLE PRECISION MHL, MHH, MH3, MH5, SH, SA
      COMMON/INPUT3/MHL,MHH,MH3,MH5,SH,SA
C==================================================================
C This initialization call must always come before anything else!
      CALL INITIALIZE_SM
C==================================================================

C================== user-defined options ==========================
C INPUTMODE = 0: take inputs from this program - enter values below
C INPUTMODE = 1: read inputs from the command line in interactive mode
      INPUTMODE = 0
C==================================================================
C Options for input sets:
C INPUTSET = 1: mu3^2, lambda1, lambda2, lambda3, lambda4, lambda5, M1, M2
C INPUTSET = 2: mu3^2, mh, lambda2, lambda3, lambda4, lambda5, M1, M2
C INPUTSET = 3: mh, mH, m3, m5, sin(thetaH), sin(alpha), M1, M2
      INPUTSET = 2
C==================================================================
C SILENT = 0: echo the inputs to the screen
C SILENT = 1: don't echo the inputs to the screen
      SILENT = 0
C==================================================================
C Modify these entries to set the input parameters (used if INPUTMODE = 0). 
      IF (INPUTMODE.EQ.0) THEN
         IF (INPUTSET.EQ.1) THEN
            MU3SQ = 90000.D0
            LAMBDA1 = 4.68D-2
            LAMBDA2 = 0.1D0
            LAMBDA3 = 0.1D0
            LAMBDA4 = 0.1D0
            LAMBDA5 = 0.1D0
            M1 = 100.D0
            M2 = 100.D0
         ELSE IF (INPUTSET.EQ.2) THEN
            MH = 125.D0
            MU3SQ = 90000.D0
            LAMBDA2 = 0.1D0
            LAMBDA3 = 0.1D0
            LAMBDA4 = 0.1D0
            LAMBDA5 = 0.1D0
            M1 = 100.D0
            M2 = 100.D0
         ELSE IF (INPUTSET.EQ.3) THEN
            MHL = 125.D0
            MHH = 288.268237D0
            MH3 = 304.221605D0
            MH5 = 339.748616D0
            SH = 0.194487374D0
            SA = -0.303281383D0
            M1 = 100.D0
            M2 = 100.D0
         ELSE
            PRINT *, "INPUTSET = ", INPUTSET, "is not a valid option."
            PRINT *, "No param_card.dat written."
            GOTO 10
         ENDIF
      ENDIF

C==================================================================
      CALL LOAD_INPUTS
      PRINT *
      IF (INPUTOK.EQ.1) THEN
         CALL THYCHECK
         IF (UNIOK*BFBOK*MINOK.EQ.1) THEN
            CALL CALCPHYS
            PRINT *, "Theory checks passed; writing param_card.dat"
            CALL WRITE_PARAM_CARD
            PRINT *
         ELSE
            PRINT *, "Theory constraint(s) failed!  ",
     .           "No param_card.dat written."
         ENDIF
      ELSE
         PRINT *, "Bad input set!  No param_card.dat written."
      ENDIF

 10   STOP
      END



